package example.localplugin;

import gridsim.gssim.DCWormsTags;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import schedframe.events.scheduling.SchedulingEvent;
import schedframe.resources.computing.ComputingNode;
import schedframe.scheduling.manager.resources.ClusterResourceManager;
import schedframe.scheduling.manager.resources.ResourceManager;
import schedframe.scheduling.manager.tasks.JobRegistry;
import schedframe.scheduling.plan.SchedulingPlanInterface;
import schedframe.scheduling.plan.impl.SchedulingPlan;
import schedframe.scheduling.plugin.grid.ModuleList;
import schedframe.scheduling.queue.TaskQueue;
import schedframe.scheduling.queue.TaskQueueList;
import schedframe.scheduling.tasks.TaskInterface;
import schedframe.scheduling.tasks.WorkloadUnit;

public class FCFSEnergyAwareClusterLocalPlugin extends BaseLocalSchedulingPlugin {

	public FCFSEnergyAwareClusterLocalPlugin () {
	}

	public SchedulingPlanInterface<?> schedule(SchedulingEvent event, TaskQueueList queues, JobRegistry jobRegistry,
			ResourceManager resManager, ModuleList modules) {

		ClusterResourceManager resourceManager = (ClusterResourceManager) resManager;
		SchedulingPlan plan = new SchedulingPlan();
		// chose the events types to serve.
		// Different actions for different events are possible.
		switch (event.getType()) {
		case START_TASK_EXECUTION:
		case TASK_FINISHED:
			// our tasks are placed only in first queue (see
			// BaseLocalPlugin.placeJobsInQueues() method)
			TaskQueue q = queues.get(0);
			// check all tasks in queue

			for (int i = 0; i < q.size(); i++) {
				WorkloadUnit job = q.get(i);
				TaskInterface<?> task = (TaskInterface<?>) job;
				// if status of the tasks in READY
				if (task.getStatus() == DCWormsTags.READY) {

					String nodeName = chooseProviderForExecution(resourceManager, task);
					if (nodeName != null) {
						addToSchedulingPlan(plan, task, nodeName);
					}
				}
			}

			break;
		}
		return plan;
	}

	private String chooseProviderForExecution(ClusterResourceManager resourceManager, TaskInterface<?> task) {
		int cpuRequest;
		try {
			cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
		} catch (NoSuchFieldException e) {
			cpuRequest = 1;
		}
		List<ComputingNode> nodes = resourceManager.getComputingNodes();
		nodes = findSuitableNodes(cpuRequest, nodes);
		Collections.sort(nodes, new Comparator<ComputingNode>(){
			 
		    public int compare(ComputingNode node1, ComputingNode node2){    
		        return node1.getResourceCharacteristic().getParameters().get("category").get(0).getContent().compareTo(node2.getResourceCharacteristic().getParameters().get("category").get(0).getContent());
		   
		    }
		});

		return nodes.size() == 0 ? null : nodes.get(0).getName();
	}
	
	private List<ComputingNode> findSuitableNodes(int cpuRequest, List<ComputingNode> nodes){
		List<ComputingNode> avNodes = new ArrayList<ComputingNode>();
		for(ComputingNode node: nodes){
			if(node.getFreeProcessorsNumber() >= cpuRequest){
				avNodes.add(node);
			}
		}
		return avNodes;
	}
	
	public String getName() {
		return getClass().getName();
	}

	class CategoryComparator implements Comparator<String>{
		 
	    public int compare(String cat1, String cat2){    
	        return cat1.compareTo(cat2);
	   
	    }
	}
}

